#!/bin/bash
set -e
INSTALLER_DIR=$(dirname "${BASH_SOURCE[0]}")
HELPER_SCRIPTS_DIR="$INSTALLER_DIR/helper_scripts"

source "$HELPER_SCRIPTS_DIR/packages.sh"

declare -A NEEDS_VERSION
NEEDS_VERSION=(
    ["$APT_VIRTUALBOX_PACKAGE"]="https://www.virtualbox.org/wiki/Linux_Downloads|${PACKAGES["$APT_VIRTUALBOX_PACKAGE"]} version (e.g. 7.1)"
    ["$RAW_GITEXTENSIONS_PACKAGE"]="https://github.com/gitextensions/gitextensions/releases|${PACKAGES["$RAW_GITEXTENSIONS_PACKAGE"]} version (e.g. 2.51.05)"
    ["$APT_PROTONVPN_PACKAGE"]="https://protonvpn.com/support/official-linux-vpn-ubuntu|${PACKAGES["$APT_PROTONVPN_PACKAGE"]} Repository version (e.g. 1.0.6)"
    ["$MAINLINE_KERNEL_PACKAGE"]="https://kernel.ubuntu.com/mainline/|Kernel versions, comma separated (e.g. 6.11.10)"
    ["$RAW_JAN_PACKAGE"]="https://github.com/janhq/jan/releases|${PACKAGES["$RAW_JAN_PACKAGE"]} version (e.g. 0.5.13)"
)

PACKAGES_WITH_GROUP=(
    "$APT_DOCKER_CE_PACKAGE"
    "$APT_VIRTUALBOX_PACKAGE"
)

declare -A BUGS
BUGS=(
    ["$FLATPAK_FLAMESHOT_PACKAGE"]="Can't take screenshot|open flameshot launcher, take screenshot from tray icon, allow permissions"
    ["$APT_VIRTUALBOX_PACKAGE"]="VirtualBox Guest Additions can't be downloaded automatically in version 7.0|Download the VirtualBox Guest Additions for your VirtualBox version from https://download.virtualbox.org/virtualbox and mount them manually into the Virtual Machine
    & Starting VM causes \`VirtualBox can't enable the AMD-V extension. Please disable the KVM kernel extension, recompile your kernel and reboot (VERR_SVM_IN_USE)\`|Add \`kvm.enable_virt_at_load=0\` to \`GRUB_CMDLINE_LINUX\` in /etc/default/grub, run \`sudo update-grub\` and reboot"
    ["$FLATPAK_NEXTCLOUD_CLIENT_PACKAGE"]="Nextcloud Client stops syncing|To debug the error, run \`flatpak run com.nextcloud.desktopclient.nextcloud\` in a terminal"
)

CAN_AUTOSTART=(
    "$SNAP_VSCODE_PACKAGE"
    "$FLATPAK_IDEA_COMMUNITY_PACKAGE"
    "$SNAP_RIDER_PACKAGE"
    "$FLATPAK_WEBSTORM_PACKAGE"
    "$FLATPAK_ANDROID_STUDIO_PACKAGE"
    "$RAW_GITEXTENSIONS_PACKAGE"
    "$FLATPAK_OBSIDIAN_PACKAGE"
    "$FLATPAK_SPOTIFY_PACKAGE"
    "$FLATPAK_GIMP_PACKAGE"
    "$APT_VIRTUALBOX_PACKAGE"
    "$APT_VLC_PACKAGE"
    "$FLATPAK_FLAMESHOT_PACKAGE"
    "$FLATPAK_NEXTCLOUD_CLIENT_PACKAGE"
    "$APT_PROTONVPN_PACKAGE"
    "$FLATPAK_FLATSEAL_PACKAGE"
    "$SNAP_FIREFOX_PACKAGE"
    "$RAW_JAN_PACKAGE"
    # "$FLATPAK_REMMINA_PACKAGE" # Adds itself to autostart
)

get_packages_from_identifier() {
    package_identifier=$1
    shift
    packages=("$@")

    for package in "${packages[@]}"; do
        echo $package | tr ' ' '\n' | grep "^$package_identifier:"
    done
}

remove_package_identifier() {
    packages=("$@")

    for package in "${packages[@]}"; do
        echo $package | sed 's/^[^:]*://'
    done
}

format_args_for_ansible() {
    args=("$@")

    for arg in "${args[@]}"; do
        echo $arg | tr '\n' ' '
    done
}

format_packages_for_ansible_from_real_package_manager() {
    package_identifier=$1
    shift
    packages=$@

    packages=$(get_packages_from_identifier "$package_identifier" "${packages[@]}")
    IFS=$'\n' read -r -d '' -a packages <<< "$packages"

    packages=$(remove_package_identifier "${packages[@]}")
    IFS=$'\n' read -r -d '' -a packages <<< "$packages"

    format_args_for_ansible "${packages[@]}"
}

main() {
    # Select packages
    selected_packages=$(for package in "${ORDERED_PACKAGES[@]}"; do
        echo "false"
        echo "${PACKAGES["$package"]}"
        echo "$package"
    done | zenity --list \
        --title="Package Selection" \
        --text="Select packages to install:" \
        --checklist \
        --column="Install" --column="Name" --column="Package" \
        --print-column "3" \
        --width=1200 --height=800 \
        2>/dev/null # suppress file descriptor error
    )

    if [[ -z "$selected_packages" ]]; then
    echo "No packages selected. Exiting."
        exit 0
    fi

    IFS="|" read -ra selected_packages <<< "$selected_packages"
    
    # Select package versions if needed
    declare -A selected_packages_need_version
    selected_packages_need_version=()
    for selected_package in "${selected_packages[@]}"; do
        if [[ -v NEEDS_VERSION["$selected_package"] ]]; then
            selected_packages_need_version+=(["$selected_package"]=true)
        else
            selected_packages_need_version+=(["$selected_package"]=false)
        fi
    done

    selected_packages_with_versions=()
    for selected_package in "${selected_packages[@]}"; do # keeps order, iterating over selected_packages_need_version has different order
        if ${selected_packages_need_version["$selected_package"]}; then
            IFS='|' read -r -a get_version_info <<< "${NEEDS_VERSION["$selected_package"]}"
            xdg-open "${get_version_info[0]}" >/dev/null 2>&1 &
            read -p "${get_version_info[1]}: " version
            selected_packages_with_versions+=("$selected_package:$version")
        else
            selected_packages_with_versions+=("$selected_package")
        fi
    done

    # Select groups from packages
    possible_user_groups_packages=()
    for selected_package in "${selected_packages[@]}"; do
        for package_with_group in "${PACKAGES_WITH_GROUP[@]}"; do
            if [[ "$selected_package" == "$package_with_group" ]]; then
                possible_user_groups_packages+=("$selected_package")
            fi
        done
    done

    user_groups=""
    if [ ${#possible_user_groups_packages[@]} -ne 0 ]; then
        user_groups=$(for possible_user_group_package in "${possible_user_groups_packages[@]}"; do
            echo "true"
            echo "${PACKAGES["$possible_user_group_package"]}"
            echo "$possible_user_group_package"
        done | zenity --list \
            --title="Group Selection" \
            --text="Select groups from packages to add the current user to:" \
            --checklist \
            --column="Add to Group" --column="Name" --column="Package" \
            --print-column "3" \
            --width=1200 --height=800 \
            2>/dev/null # suppress file descriptor error
        )

        IFS="|" read -ra user_groups <<< "$user_groups"
        user_groups=$(format_packages_for_ansible_from_real_package_manager ".*" "${user_groups[@]}")
    fi

    # Select autostart applications
    possible_autostart_applications=()
    for selected_package in "${selected_packages[@]}"; do
        for autostart_application in "${CAN_AUTOSTART[@]}"; do
            if [[ "$selected_package" == "$autostart_application" ]]; then
                possible_autostart_applications+=("$selected_package")
            fi
        done
    done

    autostart_applications=""
    if [ ${#possible_autostart_applications[@]} -ne 0 ]; then
        autostart_applications=$(for selected_package in "${possible_autostart_applications[@]}"; do
            echo "false"
            echo "${PACKAGES["$selected_package"]}"
            echo "$selected_package"
        done | zenity --list \
            --title="Autostart Selection" \
            --text="Select applications to autostart at login:" \
            --checklist \
            --column="Autostart" --column="Name" --column="Package" \
            --print-column "3" \
            --width=1200 --height=800 \
            2>/dev/null # suppress file descriptor error
        )

        IFS="|" read -ra autostart_applications <<< "$autostart_applications"
        autostart_applications=$(format_packages_for_ansible_from_real_package_manager ".*" "${autostart_applications[@]}")
    fi

    # Separate packages by source
    # apt_packages=$(extract_package_infos "$APT_IDENTIFIER" "${selected_packages_with_versions[@]}")
    apt_packages=$(format_packages_for_ansible_from_real_package_manager "$APT_IDENTIFIER" "${selected_packages_with_versions[@]}")
    snap_packages=$(format_packages_for_ansible_from_real_package_manager "$SNAP_IDENTIFIER" "${selected_packages_with_versions[@]}")
    flatpak_packages=$(format_packages_for_ansible_from_real_package_manager "$FLATPAK_IDENTIFIER" "${selected_packages_with_versions[@]}")
    raw_packages=$(format_packages_for_ansible_from_real_package_manager "$RAW_IDENTIFIER" "${selected_packages_with_versions[@]}")

    # Read mainline kernel versions
    mainline_kernels=$(format_packages_for_ansible_from_real_package_manager "$MAINLINE_KERNEL_IDENTIFIER" "${selected_packages_with_versions[@]}")
    mainline_kernels="${mainline_kernels//','/' '}"
    
    # Install packages using ansible
    bash $HELPER_SCRIPTS_DIR/install_ansible.sh
    ansible-playbook "${INSTALLER_DIR}/playbooks/install_packages.yaml" \
        --extra-vars "apt_packages='$apt_packages' snap_packages='$snap_packages' flatpak_packages='$flatpak_packages' raw_packages='$raw_packages' mainline_kernels='$mainline_kernels' user_groups='$user_groups' autostart_applications='$autostart_applications'" \
        --inventory "${INSTALLER_DIR}/ansible_inventory.ini" \
        --ask-become-pass

    # Print bugs and possible resolutions
    for selected_package in "${selected_packages[@]}"; do
        if [[ -v BUGS[$selected_package] ]]; then
            IFS='&' read -r -a bugs <<< $(echo "${BUGS[$selected_package]}" | tr -d '\n')
            for bug in "${bugs[@]}"; do
                read -r bug <<< "$bug"
                IFS='|' read -r -a bug_infos <<< "$bug"
                echo "Bug in package ${PACKAGES[$selected_package]} ($selected_package): ${bug_infos[0]}. Suggested solution: ${bug_infos[1]}."
            done
            echo ''
        fi
    done
}

main
